@@ -2,10 +2,16 @@ import numpy as np
 
 from . import lib
 from . import Geometry  # NOQA
-
-__all__ = ["area", "distance", "bounds", "total_bounds", "length", "hausdorff_distance"]
+from .decorators import requires_geos, multithreading_enabled
 
 
+__all__ = [
+    "area", "distance", "bounds", "total_bounds", "length", "hausdorff_distance",
+    "frechet_distance", "minimum_clearance"
+]
+
+
+@multithreading_enabled
 def area(geometry, **kwargs):
     """Computes the area of a (multi)polygon.
 
@@ -27,6 +33,7 @@ def area(geometry, **kwargs):
     return lib.area(geometry, **kwargs)
 
 
+@multithreading_enabled
 def distance(a, b, **kwargs):
     """Computes the Cartesian distance between two geometries.
 
@@ -51,6 +58,7 @@ def distance(a, b, **kwargs):
     return lib.distance(a, b, **kwargs)
 
 
+@multithreading_enabled
 def bounds(geometry, **kwargs):
     """Computes the bounds (extent) of a geometry.
 
@@ -73,7 +81,7 @@ def bounds(geometry, **kwargs):
     """
     # We need to provide the `out` argument here for compatibility with
     # numpy < 1.16. See https://github.com/numpy/numpy/issues/14949
-    geometry_arr = np.asarray(geometry, dtype=np.object)
+    geometry_arr = np.asarray(geometry, dtype=np.object_)
     out = np.empty(geometry_arr.shape + (4,), dtype="float64")
     return lib.bounds(geometry_arr, out=out, **kwargs)
 
@@ -117,6 +125,7 @@ def total_bounds(geometry, **kwargs):
     )
 
 
+@multithreading_enabled
 def length(geometry, **kwargs):
     """Computes the length of a (multi)linestring or polygon perimeter.
 
@@ -140,10 +149,11 @@ def length(geometry, **kwargs):
     return lib.length(geometry, **kwargs)
 
 
+@multithreading_enabled
 def hausdorff_distance(a, b, densify=None, **kwargs):
-    """Compute the discrete Haussdorf distance between two geometries.
+    """Compute the discrete Hausdorff distance between two geometries.
 
-    The Haussdorf distance is a measure of similarity: it is the greatest
+    The Hausdorff distance is a measure of similarity: it is the greatest
     distance between any point in A and the closest point in B. The discrete
     distance is an approximation of this metric: only vertices are considered.
     The parameter 'densify' makes this approximation less coarse by splitting
@@ -171,4 +181,70 @@ def hausdorff_distance(a, b, densify=None, **kwargs):
     if densify is None:
         return lib.hausdorff_distance(a, b, **kwargs)
     else:
-        return lib.haussdorf_distance_densify(a, b, densify, **kwargs)
+        return lib.hausdorff_distance_densify(a, b, densify, **kwargs)
+
+
+@requires_geos("3.7.0")
+@multithreading_enabled
+def frechet_distance(a, b, densify=None, **kwargs):
+    """Compute the discrete Fréchet distance between two geometries.
+
+    The Fréchet distance is a measure of similarity: it is the greatest
+    distance between any point in A and the closest point in B. The discrete
+    distance is an approximation of this metric: only vertices are considered.
+    The parameter 'densify' makes this approximation less coarse by splitting
+    the line segments between vertices before computing the distance.
+
+    Fréchet distance sweep continuously along their respective curves
+    and the direction of curves is significant. This makes it a better measure
+    of similarity than Hausdorff distance for curve or surface matching.
+
+    Parameters
+    ----------
+    a, b : Geometry or array_like
+    densify : float, array_like or None
+        The value of densify is required to be between 0 and 1.
+
+    Examples
+    --------
+    >>> line_1 = Geometry("LINESTRING (0 0, 100 0)")
+    >>> line_2 = Geometry("LINESTRING (0 0, 50 50, 100 0)")
+    >>> frechet_distance(line_1, line_2)  # doctest: +ELLIPSIS
+    70.71...
+    >>> frechet_distance(line_1, line_2, densify=0.5)
+    50.0
+    >>> frechet_distance(line_1, Geometry("LINESTRING EMPTY"))
+    nan
+    >>> frechet_distance(line_1, None)
+    nan
+    """
+    if densify is None:
+        return lib.frechet_distance(a, b, **kwargs)
+    return lib.frechet_distance_densify(a, b, densify, **kwargs)
+
+
+@requires_geos("3.6.0")
+@multithreading_enabled
+def minimum_clearance(geometry, **kwargs):
+    """Computes the Minimum Clearance distance.
+
+    A geometry's "minimum clearance" is the smallest distance by which
+    a vertex of the geometry could be moved to produce an invalid geometry.
+
+    If no minimum clearance exists for a geometry (for example, a single
+    point, or an empty geometry), infinity is returned.
+
+    Parameters
+    ----------
+    geometry : Geometry or array_like
+
+    Examples
+    --------
+    >>> minimum_clearance(Geometry("POLYGON((0 0, 0 10, 5 6, 10 10, 10 0, 5 4, 0 0))"))
+    2.0
+    >>> minimum_clearance(Geometry("POLYGON EMPTY"))
+    inf
+    >>> minimum_clearance(None)
+    nan
+    """
+    return lib.minimum_clearance(geometry, **kwargs)
