@@ -2,14 +2,16 @@ import warnings
 
 from . import lib
 from . import Geometry  # NOQA
-from .decorators import multithreading_enabled
+from .decorators import multithreading_enabled, requires_geos
 
 __all__ = [
     "has_z",
+    "is_ccw",
     "is_closed",
     "is_empty",
     "is_geometry",
     "is_missing",
+    "is_prepared",
     "is_ring",
     "is_simple",
     "is_valid",
@@ -27,6 +29,7 @@ __all__ = [
     "within",
     "equals_exact",
     "relate",
+    "relate_pattern",
 ]
 
 
@@ -48,6 +51,41 @@ def has_z(geometry, **kwargs):
     return lib.has_z(geometry, **kwargs)
 
 
+@requires_geos("3.7.0")
+@multithreading_enabled
+def is_ccw(geometry, **kwargs):
+    """Returns True if a linestring or linearring is counterclockwise.
+
+    Note that there are no checks on whether lines are actually closed and
+    not self-intersecting, while this is a requirement for is_ccw. The recommended
+    usage of this function for linestrings is ``is_ccw(g) & is_simple(g)`` and for
+    linearrings ``is_ccw(g) & is_valid(g)``.
+
+    Parameters
+    ----------
+    geometry : Geometry or array_like
+        This function will return False for non-linear goemetries and for
+        lines with fewer than 4 points (including the closing point).
+
+    See also
+    --------
+    is_simple : Checks if a linestring is closed and simple.
+    is_valid : Checks additionally if the geometry is simple.
+
+    Examples
+    --------
+    >>> is_ccw(Geometry("LINEARRING (0 0, 0 1, 1 1, 0 0)"))
+    False
+    >>> is_ccw(Geometry("LINEARRING (0 0, 1 1, 0 1, 0 0)"))
+    True
+    >>> is_ccw(Geometry("LINESTRING (0 0, 1 1, 0 1)"))
+    False
+    >>> is_ccw(Geometry("POINT (0 0)"))
+    False
+    """
+    return lib.is_ccw(geometry, **kwargs)
+
+
 @multithreading_enabled
 def is_closed(geometry, **kwargs):
     """Returns True if a linestring's first and last points are equal.
@@ -97,7 +135,7 @@ def is_empty(geometry, **kwargs):
     """
     return lib.is_empty(geometry, **kwargs)
 
-
+@multithreading_enabled
 def is_geometry(geometry, **kwargs):
     """Returns True if the object is a geometry
 
@@ -123,7 +161,7 @@ def is_geometry(geometry, **kwargs):
     """
     return lib.is_geometry(geometry, **kwargs)
 
-
+@multithreading_enabled
 def is_missing(geometry, **kwargs):
     """Returns True if the object is not a geometry (None)
 
@@ -151,6 +189,40 @@ def is_missing(geometry, **kwargs):
     return lib.is_missing(geometry, **kwargs)
 
 
+@multithreading_enabled
+def is_prepared(geometry, **kwargs):
+    """Returns True if a Geometry is prepared.
+
+    Note that it is not necessary to check if a geometry is already prepared
+    before preparing it. It is more efficient to call ``prepare`` directly
+    because it will skip geometries that are already prepared.
+
+    This function will return False for missing geometries (None).
+
+    Parameters
+    ----------
+    geometry : Geometry or array_like
+
+    See also
+    --------
+    is_valid_input : check if an object is a geometry or None
+    prepare : prepare a geometry
+
+    Examples
+    --------
+    >>> geometry = Geometry("POINT (0 0)")
+    >>> is_prepared(Geometry("POINT (0 0)"))
+    False
+    >>> from pygeos import prepare; prepare(geometry);
+    >>> is_prepared(geometry)
+    True
+    >>> is_prepared(None)
+    False
+    """
+    return lib.is_prepared(geometry, **kwargs)
+
+
+@multithreading_enabled
 def is_valid_input(geometry, **kwargs):
     """Returns True if the object is a geometry or None
 
@@ -215,6 +287,9 @@ def is_simple(geometry, **kwargs):
     """Returns True if a Geometry has no anomalous geometric points, such as
     self-intersections or self tangency.
 
+    Note that polygons and linearrings are assumed to be simple. Use is_valid
+    to check these kind of geometries for self-intersections.
+
     Parameters
     ----------
     geometry : Geometry or array_like
@@ -223,6 +298,7 @@ def is_simple(geometry, **kwargs):
     See also
     --------
     is_ring : Checks additionally if the geometry is closed.
+    is_valid : Checks whether a geometry is well formed.
 
     Examples
     --------
@@ -305,6 +381,10 @@ def crosses(a, b, **kwargs):
     ----------
     a, b : Geometry or array_like
 
+    See also
+    --------
+    prepare : improve performance by preparing ``a`` (the first argument)
+
     Examples
     --------
     >>> line = Geometry("LINESTRING(0 0, 1 1)")
@@ -343,6 +423,7 @@ def contains(a, b, **kwargs):
     See also
     --------
     within : ``contains(A, B) == within(B, A)``
+    prepare : improve performance by preparing ``a`` (the first argument)
 
     Examples
     --------
@@ -384,6 +465,7 @@ def covered_by(a, b, **kwargs):
     See also
     --------
     covers : ``covered_by(A, B) == covers(B, A)``
+    prepare : improve performance by preparing ``a`` (the first argument)
 
     Examples
     --------
@@ -425,6 +507,7 @@ def covers(a, b, **kwargs):
     See also
     --------
     covered_by : ``covers(A, B) == covered_by(B, A)``
+    prepare : improve performance by preparing ``a`` (the first argument)
 
     Examples
     --------
@@ -469,6 +552,7 @@ def disjoint(a, b, **kwargs):
     See also
     --------
     intersects : ``disjoint(A, B) == ~intersects(A, B)``
+    prepare : improve performance by preparing ``a`` (the first argument)
 
     Examples
     --------
@@ -534,6 +618,7 @@ def intersects(a, b, **kwargs):
     See also
     --------
     disjoint : ``intersects(A, B) == ~disjoint(A, B)``
+    prepare : improve performance by preparing ``a`` (the first argument)
 
     Examples
     --------
@@ -559,6 +644,10 @@ def overlaps(a, b, **kwargs):
     ----------
     a, b : Geometry or array_like
 
+    See also
+    --------
+    prepare : improve performance by preparing ``a`` (the first argument)
+
     Examples
     --------
     >>> line = Geometry("LINESTRING(0 0, 1 1)")
@@ -585,6 +674,10 @@ def touches(a, b, **kwargs):
     ----------
     a, b : Geometry or array_like
 
+    See also
+    --------
+    prepare : improve performance by preparing ``a`` (the first argument)
+
     Examples
     --------
     >>> line = Geometry("LINESTRING(0 2, 2 0)")
@@ -623,6 +716,7 @@ def within(a, b, **kwargs):
     See also
     --------
     contains : ``within(A, B) == contains(B, A)``
+    prepare : improve performance by preparing ``a`` (the first argument)
 
     Examples
     --------
@@ -710,3 +804,33 @@ def relate(a, b, **kwargs):
     'F0FFFF102'
     """
     return lib.relate(a, b, **kwargs)
+
+
+@multithreading_enabled
+def relate_pattern(a, b, pattern, **kwargs):
+    """
+    Returns True if the DE-9IM string code for the relationship between
+    the geometries satisfies the pattern, else False.
+
+    This function compares the DE-9IM code string for two geometries
+    against a specified pattern. If the string matches the pattern then
+    ``True`` is returned, otherwise ``False``. The pattern specified can
+    be an exact match (``0``, ``1`` or ``2``), a boolean match
+    (uppercase ``T`` or ``F``), or a wildcard (``*``). For example,
+    the pattern for the `within` predicate is ``'T*F**F***'``.
+
+    Parameters
+    ----------
+    a, b : Geometry or array_like
+    pattern : string
+
+    Examples
+    --------
+    >>> point = Geometry("POINT (0.5 0.5)")
+    >>> square = Geometry("POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))")
+    >>> relate(point, square)
+    '0FFFFF212'
+    >>> relate_pattern(point, square, "T*F**F***")
+    True
+    """
+    return lib.relate_pattern(a, b, pattern, **kwargs)
